/* This source file is a part of the GePhex Project.

 Copyright (C) 2001-2004
 
 Georg Seidel <georg@gephex.org> 
 Martin Bayer <martin@gephex.org> 
 Phillip Promesberger <coma@gephex.org>
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
   
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
	
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#include "v4lcapturedriver.h"

#include <string>
#include <stdexcept>

#include <cassert>

//#include "libscale.h"


//----------------------------------------------------------------------

class V4LCaptureDriver::Impl
{
public:
  Impl() {}
  ~Impl()  {}
	
  void close()  						
  {
    throw std::logic_error("NOT IMPLEMENTED YET");
  }
	
  void open(int device_num)
  {
    throw std::logic_error("NOT IMPLEMENTED YET");
  }

  void grab_frame(uint_32* frb, int width, int height)
  {
    throw std::logic_error("NOT IMPLEMENTED YET");
  }
  
  bool is_open() const
  {
    return false;
  }
  
  int width() const { return 0; }

  int height() const { return 0; }
  
  bool new_sample() const { return false; }

private:
};

//----------------------------------------------------------------------

V4LCaptureDriver::V4LCaptureDriver()
  : m_impl(new Impl())
{
}

V4LCaptureDriver::~V4LCaptureDriver()
{
}


void V4LCaptureDriver::open(int device)
{
  if (m_impl->is_open())
    throw std::logic_error("Device already open");
	
  m_impl->open(device);
}

void V4LCaptureDriver::close()
{
  if (m_impl->is_open())
    m_impl->close();
}

void V4LCaptureDriver::decode_frame(uint_32* framebuffer,
                                    int width, int height)
{
  if (!m_impl->is_open())
    throw std::logic_error("Device not open");
	
  m_impl->grab_frame(framebuffer, width, height);
}

void V4LCaptureDriver::status(int& width,
                              int& height,
                              bool& new_sample) const
{
  width      = m_impl->width();
  height     = m_impl->height();
  new_sample = m_impl->new_sample();
}

bool V4LCaptureDriver::is_open() const
{
  return m_impl->is_open();
}
