/* This source file is a part of the GePhex Project.

 Copyright (C) 2001-2004

 Georg Seidel <georg@gephex.org> 
 Martin Bayer <martin@gephex.org> 
 Phillip Promesberger <coma@gephex.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#ifndef INCLUDED_VIDEODEVICE_H
#define INCLUDED_VIDEODEVICE_H

#include <string>
#include <stdexcept>

#include "basic_types.h"
#include "v4l1.h"

class VideoDevice
{
 public:
  typedef void (*logT) (int, const char*);
  
  enum PixelType
    {
      RGBA32,
      YUV411P
    };
  
  struct Frame
  {
    int xSize;
    int ySize;
    unsigned int type;
    uint_32* frameptr;
  };
  
  
  /** opens a V4L videodevice
   * \param deviceFileName absolute path of the devicefile e.g. "/dev/video1"
   */
  VideoDevice(const std::string& deviceFileName, logT log2) throw (std::runtime_error);

  size_t min_width();
  size_t max_width();
  size_t min_height();
  size_t max_height();
  
  void initialize(const Frame& frame);

  bool is_initialized();  

  size_t width();
  size_t height();


  
  void setProperties(int brightness,int hue,int colour,
		     int contrast,
		     int whiteness);
  
  /** grabs one frame 
   * \param frame image params 
   */
  void grabImage(Frame& frame,bool drop) throw (std::runtime_error);

  void uninitialize();

  /** closes a V4L videodevice
   */
  ~VideoDevice();

private:
  /**
   * Video4Linux 1 device
   */
  v4l1_device vd;



  /**
   * struct will be filled with capture info
   */
  video_capability cap; 
  
  bool initialized;
  Frame initFrame;
  
  // converter parameters
  int m_brightness;
  int m_hue;
  int m_colour;
  int m_contrast;
  int m_whiteness;

  size_t m_height;
  size_t m_width;
  unsigned int m_pixelformat;

  // mmap mode
  void* mmapBase;
  size_t mmapSize;
  uint_32* frameptr[2];
  int frontbf;
  int backbf;

  bool m_can_select;

  logT log2;
};

#endif



